@@ -183,7 +183,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 
     if(parent->children == NULL) {
         /* TODO: add indents */
-	text = xmlNewText(BAD_CAST "\n"); 
+	text = xmlNewText(xmlSecStringCR); 
         if(text == NULL) {	
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -209,15 +209,16 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
     if(ns != NULL) {
 	xmlNsPtr nsPtr;
 	
-	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
-	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
+        /* find namespace by href and check that its prefix is not overwritten */
+	nsPtr = xmlSearchNsByHref(cur->doc, cur, ns);
+	if((nsPtr == NULL) || (xmlSearchNs(cur->doc, cur, nsPtr->prefix) != nsPtr)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
     
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n"); 
+    text = xmlNewText(xmlSecStringCR); 
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -249,7 +250,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
 
     if(parent->children == NULL) {
         /* TODO: add indents */
-	text = xmlNewText(BAD_CAST "\n"); 
+	text = xmlNewText(xmlSecStringCR); 
         if(text == NULL) {	
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -264,7 +265,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
     xmlAddChild(parent, child);
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n"); 
+    text = xmlNewText(xmlSecStringCR); 
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -311,15 +312,16 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     if(ns != NULL) {
 	xmlNsPtr nsPtr;
 	
-	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
-	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
+        /* find namespace by href and check that its prefix is not overwritten */
+	nsPtr = xmlSearchNsByHref(cur->doc, cur, ns);
+	if((nsPtr == NULL) || (xmlSearchNs(cur->doc, cur, nsPtr->prefix) != nsPtr)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n");
+    text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -366,15 +368,16 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     if(ns != NULL) {
 	xmlNsPtr nsPtr;
 	
-	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
-	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
+        /* find namespace by href and check that its prefix is not overwritten */
+	nsPtr = xmlSearchNsByHref(cur->doc, cur, ns);
+	if((nsPtr == NULL) || (xmlSearchNs(cur->doc, cur, nsPtr->prefix) != nsPtr)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n");
+    text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -416,6 +419,21 @@ xmlSecGetNextElementNode(xmlNodePtr cur) {
  */
 int
 xmlSecReplaceNode(xmlNodePtr node, xmlNodePtr newNode) {
+    return xmlSecReplaceNodeAndReturn(node, newNode, NULL);
+}
+
+/**                 
+ * xmlSecReplaceNodeAndReturn:
+ * @node: 		the current node.
+ * @newNode: 		the new node.
+ * @replaced:   	the replaced node, or release it if NULL is given
+ * 
+ * Swaps the @node and @newNode in the XML tree.
+ *
+ * Returns 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecReplaceNodeAndReturn(xmlNodePtr node, xmlNodePtr newNode, xmlNodePtr* replaced) {
     xmlNodePtr oldNode;
     int restoreRoot = 0;
     
@@ -445,7 +463,13 @@ xmlSecReplaceNode(xmlNodePtr node, xmlNodePtr newNode) {
 	xmlDocSetRootElement(oldNode->doc, newNode);
     }
 
-    xmlFreeNode(oldNode);
+    /* return the old node if requested */
+    if(replaced != NULL) {
+     	(*replaced) = oldNode;  	  	
+    } else {
+   	xmlFreeNode(oldNode); 
+    }
+   
     return(0);
 }
 
@@ -460,19 +484,55 @@ xmlSecReplaceNode(xmlNodePtr node, xmlNodePtr newNode) {
  */
 int
 xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
+     return xmlSecReplaceContentAndReturn(node, newNode, NULL);
+}
+
+/**
+ * xmlSecReplaceContentAndReturn
+ * @node: 		the current node.
+ * @newNode: 		the new node.
+ * @replaced:   	the replaced nodes, or release them if NULL is given
+ * 
+ * Swaps the content of @node and @newNode.
+ *
+ * Returns 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecReplaceContentAndReturn(xmlNodePtr node, xmlNodePtr newNode, xmlNodePtr *replaced) {
     xmlSecAssert2(node != NULL, -1);
     xmlSecAssert2(newNode != NULL, -1);  
 
     xmlUnlinkNode(newNode);
     xmlSetTreeDoc(newNode, node->doc);
-    xmlNodeSetContent(node, NULL);
+
+    /* return the old nodes if requested */
+    if(replaced != NULL) {
+        xmlNodePtr cur, next, tail;
+
+        (*replaced) = tail = NULL;
+        for(cur = node->children; (cur != NULL); cur = next) {
+            next = cur->next;
+            if((*replaced) != NULL) {
+		/* n is unlinked in this function */        	
+                xmlAddNextSibling(tail, cur); 
+		tail = cur;
+            } else {
+		/* this is the first node, (*replaced) is the head */
+                xmlUnlinkNode(cur);
+    	        (*replaced) = tail = cur;    	    
+          }
+        }
+    } else {
+	/* just delete the content */
+        xmlNodeSetContent(node, NULL);
+    }
+
     xmlAddChild(node, newNode);
     xmlSetTreeDoc(newNode, node->doc);
 
     return(0);
 }
 
-
 /**
  * xmlSecReplaceNodeBuffer:
  * @node: 		the current node.
@@ -484,49 +544,93 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
  * Returns 0 on success or a negative value if an error occurs.
  */
 int
-xmlSecReplaceNodeBuffer(xmlNodePtr node, 
-			const xmlSecByte *buffer, xmlSecSize size) {
-    static const char dummyPrefix[] = "<dummy>";
-    static const char dummyPostfix[] = "</dummy>";
-    xmlDocPtr doc;
-    xmlNodePtr ptr1, ptr2;
+xmlSecReplaceNodeBuffer(xmlNodePtr node, const xmlSecByte *buffer, xmlSecSize size) {
+    return xmlSecReplaceNodeBufferAndReturn(node, buffer, size, NULL);
+}
+
+/**
+ * xmlSecReplaceNodeBufferAndReturn:
+ * @node: 		the current node.
+ * @buffer: 		the XML data.
+ * @size: 		the XML data size.
+ * @replaced: 		the replaced nodes, or release them if NULL is given
+ * 
+ * Swaps the @node and the parsed XML data from the @buffer in the XML tree.
+ *
+ * Returns 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecReplaceNodeBufferAndReturn(xmlNodePtr node, const xmlSecByte *buffer, xmlSecSize size, xmlNodePtr *replaced) {
+    xmlNodePtr results = NULL;
+    xmlNodePtr next = NULL;
 
     xmlSecAssert2(node != NULL, -1);
-    
-    doc = xmlSecParseMemoryExt((xmlSecByte*)dummyPrefix, strlen(dummyPrefix),
-			       buffer, size,
-			       (xmlSecByte*)dummyPostfix, strlen(dummyPostfix));
-    if(doc == NULL){
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlSecParseMemoryExt",
-		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
-	return(-1);	    	
+    xmlSecAssert2(node->parent != NULL, -1);
+
+    /* parse buffer in the context of node's parent */
+    if(xmlParseInNodeContext(node->parent, (const char*)buffer, size, XML_PARSE_NODICT, &results) != XML_ERR_OK) {
+        xmlSecError(XMLSEC_ERRORS_HERE,
+                    NULL,
+                    "xmlParseInNodeContext",
+                    XMLSEC_ERRORS_R_XML_FAILED,
+                    "Failed to parse content");
+        return(-1);         
     }
-	    
-    ptr1 = xmlDocGetRootElement(doc);
-    if(ptr1 == NULL){
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlDocGetRootElement",
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "root is null");
-	xmlFreeDoc(doc);
-	return(-1);	    	
+
+    /* add new nodes */
+    while (results != NULL) {
+        next = results->next;
+        xmlAddPrevSibling(node, results);
+        results = next;
     }
-    
-    ptr1 = ptr1->children;
-    while(ptr1 != NULL) {
-	ptr2 = ptr1->next;
-	xmlUnlinkNode(ptr1);
-	xmlAddPrevSibling(node, ptr1);
-	ptr1 = ptr2;
-    }
-	    
+
+    /* remove old node */
     xmlUnlinkNode(node);
-    xmlFreeNode(node);  
-    xmlFreeDoc(doc);
+
+    /* return the old node if requested */
+    if(replaced != NULL) {
+     	(*replaced) = node;  	  	
+    } else {
+   	xmlFreeNode(node); 
+    }
+
+    return(0);
+}
+
+/**
+ * xmlSecNodeEncodeAndSetContent:
+ * @node: 		    the pointer to an XML node.
+ * @buffer: 		the pointer to the node content.
+ *
+ * Encodes "special" characters in the @buffer and sets the result
+ * as the node content.
+ *
+ * Returns 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecNodeEncodeAndSetContent(xmlNodePtr node, const xmlChar * buffer) {
+    xmlSecAssert2(node != NULL, -1);
+    xmlSecAssert2(node->doc != NULL, -1);
+    
+    if(buffer != NULL) {
+	    xmlChar * tmp;
+
+        tmp = xmlEncodeSpecialChars(node->doc, buffer);        
+        if (tmp == NULL) {
+            xmlSecError(XMLSEC_ERRORS_HERE,
+                        NULL,
+                        "xmlEncodeSpecialChars",
+                        XMLSEC_ERRORS_R_XML_FAILED,
+                        "Failed to encode special characters");
+            return(-1);         
+        }
+
+        xmlNodeSetContent(node, tmp);
+        xmlFree(tmp);
+    } else {
+        xmlNodeSetContent(node, NULL);
+    }
+
     return(0);
 }
 
@@ -631,6 +735,7 @@ xmlSecGenerateAndAddID(xmlNodePtr node, const xmlChar* attrName, const xmlChar*
 	        return(-1);	
 	    }
 	    
+	    xmlFree(id);
 	    return(0);
 	}
 	xmlFree(id);
@@ -723,7 +828,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 	
 	tmpLen = xmlStrlen(prefix) + xmlStrlen(res) + 1;
 	tmp = xmlMalloc(tmpLen + 1);
-	if(res == NULL) {
+	if(tmp == NULL) {
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 		        NULL,
 			"xmlMalloc",
@@ -733,7 +838,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 	    return(NULL);
 	}
 	
-	xmlSecStrPrintf(res, tmpLen, BAD_CAST "%s%s", prefix, res);
+	xmlSecStrPrintf(tmp, tmpLen, BAD_CAST "%s%s", prefix, res);
 	xmlFree(res);
 	res = tmp;    
     } else {
